Method: Pathutil.encoding

Defined in:
lib/pathutil.rb

.encodingObject


Note: you are encouraged to override this if you need to. Aliases the default system encoding to us so that we can do most read and write operations with that encoding, instead of being crazy.




633
634
635
636
637
# File 'lib/pathutil.rb', line 633

def encoding
  return @encoding ||= begin
    Encoding.default_external
  end
end